@@ -41,11 +41,11 @@ const Sidebar = new Lang.Class({
     Extends: Gtk.Revealer,
 
     _init: function(mapView) {
-        this.parent({ visible:             true,
-                      transition_type:     Gtk.RevealerTransitionType.SLIDE_LEFT,
+        this.parent({ visible: true,
+                      transition_type: Gtk.RevealerTransitionType.SLIDE_LEFT,
                       transition_duration: 400, // ms
-                      halign:              Gtk.Align.END,
-                      valign:              Gtk.Align.FILL
+                      halign: Gtk.Align.END,
+                      valign: Gtk.Align.FILL
                     });
         this.get_style_context().add_class('maps-sidebar');
 
@@ -79,9 +79,13 @@ const Sidebar = new Lang.Class({
                                         ui.modeBikeToggle,
                                         ui.modeCarToggle);
 
-        this._initRouteEntry(ui.fromEntryGrid, 0);
+        let fromEntry = this._initRouteEntry(ui.fromEntryGrid, 0);
         this._initRouteEntry(ui.toEntryGrid, 1);
 
+        this.bind_property('child-revealed',
+                           fromEntry, 'has_focus',
+                           GObject.BindingFlags.DEFAULT);
+
         ui.viaAddButton.connect('clicked', (function() {
             this._createViaRow(ui.viaGridContainer);
         }).bind(this));
@@ -101,7 +105,7 @@ const Sidebar = new Lang.Class({
         car.connect('toggled', onToggle.bind(this, transport.CAR));
         bike.connect('toggled', onToggle.bind(this, transport.BIKE));
 
-        query.connect('notify::transportation', function() {
+        let setToggles = function() {
             switch(query.transportation) {
             case transport.PEDESTRIAN:
                 pedestrian.active = true;
@@ -113,16 +117,19 @@ const Sidebar = new Lang.Class({
                 bike.active = true;
                 break;
             }
-        });
+        };
+
+        setToggles();
+        query.connect('notify::transportation', setToggles);
     },
 
     _createPlaceEntry: function() {
         return new PlaceEntry.PlaceEntry({ visible: true,
                                            can_focus: true,
+                                           hexpand: true,
                                            receives_default: true,
-                                           margin_start: 5,
-                                           width_request: 220,
-                                           mapView: this._mapView });
+                                           mapView: this._mapView,
+                                           parseOnFocusOut: true });
     },
 
     _createViaRow: function(listbox) {
@@ -153,6 +160,8 @@ const Sidebar = new Lang.Class({
                             point, 'place',
                             GObject.BindingFlags.BIDIRECTIONAL);
         Application.routeService.query.addPoint(point, pointIndex);
+
+        return entry;
     },
 
     _initInstructionList: function() {
@@ -166,14 +175,13 @@ const Sidebar = new Lang.Class({
                 query.removePoint(row.get_index() + 1);
                 row.destroy();
             }).bind(this));
-
-            this._timeInfo.label = '';
-            this._distanceInfo.label = '';
         }).bind(this));
 
         query.connect('notify', (function() {
             if (query.isValid())
                 this._instructionStack.visible_child = this._instructionSpinner;
+            else
+                this._clearInstructions();
         }).bind(this));
 
         route.connect('update', (function() {
@@ -200,6 +208,9 @@ const Sidebar = new Lang.Class({
     _clearInstructions: function() {
         let listBox = this._instructionList;
         listBox.forall(listBox.remove.bind(listBox));
+
+        this._timeInfo.label = '';
+        this._distanceInfo.label = '';
     }
 });
 
